# -*- coding: utf-8 -*-
#
#   Copyright (C) 2010 Wei J <wei.jiang34@gmail.com>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License version 2,
#   or (at your option) any later version, as published by the Free
#   Software Foundation
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the
#   Free Software Foundation, Inc.,
#   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#

import os
import re
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from PyQt4 import uic
from PyKDE4.plasma import Plasma
from PyKDE4 import plasmascript
from PyKDE4.kdecore import *
from PyKDE4.kdeui import *

import moc
import mymoc

class MocPlasmaClient(plasmascript.Applet):
    def __init__(self,parent,args=None):
        plasmascript.Applet.__init__(self,parent)
        
    def init(self):
        self.setHasConfigurationInterface(True)
        
        self.theme = Plasma.Svg(self)
        self.theme.setImagePath("widgets/background")
        self.setBackgroundHints(Plasma.Applet.DefaultBackground)
        self.setAspectRatioMode(Plasma.IgnoreAspectRatio)

        if mymoc.isInstalled() == 1:
            pass
            
        self.general_config = self.config("General")
        self.fromGeneralConfig()
        
        self.horizon = 'true'
        self.systemIcon = 'true'
        
        self.setupInterface(self.horizon, self.systemIcon)
        
        self.timer = QTimer();
        QObject.connect(self.timer, SIGNAL("timeout()"), self.checkRunning)
        self.timer.start(1000);

# configuration interface
    def fromGeneralConfig(self):
        self.systemIcon = self.general_config.readEntry("sysicon", QString('true')).toString()
        self.userIcon = self.general_config.readEntry("usericon", QString('false')).toString()
        self.showpopup = self.general_config.readEntry("showpopup", QString('true')).toString()
        self.horizon = self.general_config.readEntry("horizon", QString('true')).toString()
        self.vertical = self.general_config.readEntry("vertical", QString('false')).toString()
    
    def configChanged(self):
        self.fromGeneralConfig()
        plasmascript.Applet.configChanged(self)
        self.removeIntarface()
        self.setupInterface(self.horizon, self.systemIcon)
        
# main interface
    def setupInterface(self, o, i):
        self.setupOrientation(o)
        self.setupIcons(i)

        #=== Previous Button ===
        self.prevButton = Plasma.IconWidget(self.prevIcon, "", self.applet)
        #self.prevButton.setMaximumWidth(36)
        #self.prevButton.setMaximumHeight(36)
        self.prevButton.setContentsMargins(0, 0, 0, 0)

        #=== Play/Pause button ===
        self.ppButton = Plasma.IconWidget(self.playIcon, "", self.applet)
        self.ppButton.setContentsMargins(0, 0, 0, 0)

        #=== Stop Button ===
        self.stopButton = Plasma.IconWidget(self.stopIcon, "", self.applet)
        self.stopButton.setContentsMargins(0, 0, 0, 0)

        #=== Next Button ===
        self.nextButton = Plasma.IconWidget(self.nextIcon, "", self.applet)
        self.nextButton.setContentsMargins(0, 0, 0, 0)

        #=== ConnectButton ===
        self.connectButton = Plasma.IconWidget(self.connectIcon, "", self.applet)
        self.connectButton.setContentsMargins(0, 0, 0, 0)
        
        if mymoc.isRunning() == 1:
            self.addButtons()
            self.applet.setLayout(self.buttonsLayout)
        else:
            self.removeButtons()
            self.applet.setLayout(self.buttonsLayout)
        
        QObject.connect(self.prevButton, SIGNAL("clicked()"), self.previous)
        QObject.connect(self.prevButton, SIGNAL("clicked()"), self.changeIcon)
        QObject.connect(self.ppButton, SIGNAL("clicked()"), self.togglePlayback)
        QObject.connect(self.ppButton, SIGNAL("clicked()"), self.changeIcon)
        QObject.connect(self.stopButton, SIGNAL("clicked()"), self.stop)
        QObject.connect(self.stopButton, SIGNAL("clicked()"), self.changeIcon)
        QObject.connect(self.nextButton, SIGNAL("clicked()"), self.next)
        QObject.connect(self.nextButton, SIGNAL("clicked()"), self.changeIcon)
        QObject.connect(self.connectButton, SIGNAL("clicked()"), self.addButtons)
        QObject.connect(self.connectButton, SIGNAL("clicked()"), self.startMocp)  
        
    def setupOrientation(self, orientation):
        if orientation == 'true':
            self.buttonsLayout = QGraphicsLinearLayout(Qt.Horizontal, self.applet)
        else:
            self.buttonsLayout = QGraphicsLinearLayout(Qt.Vertical, self.applet)
            
    def setupIcons(self, iconTheme):
        if iconTheme == 'false':
            self.prevIcon = QIcon(unicode(self.package().path()) + "contents/icons/prev.png")
            self.playIcon = QIcon(unicode(self.package().path()) + "contents/icons/play.png")
            self.pauseIcon = QIcon(unicode(self.package().path()) + "contents/icons/pause.png")
            self.stopIcon = QIcon(unicode(self.package().path()) + "contents/icons/stop.png")
            self.nextIcon = QIcon(unicode(self.package().path()) + "contents/icons/next.png")
            self.connectIcon = QIcon(unicode(self.package().path()) + "contents/icons/connect.png")
        else:
            self.prevIcon = QIcon(KIcon("media-skip-backward"))
            self.playIcon = QIcon(KIcon("media-playback-start"))
            self.pauseIcon = QIcon(KIcon("media-playback-pause"))
            self.nextIcon = QIcon(KIcon("media-skip-forward"))
            self.stopIcon = QIcon(KIcon("media-playback-stop"))
            self.connectIcon = QIcon(KIcon("media-record"))

    def addButtons(self):
        self.buttonsLayout.removeItem(self.connectButton)
        self.connectButton.hide()
        self.buttonsLayout.addItem(self.prevButton)
        self.prevButton.show()
        self.buttonsLayout.addItem(self.ppButton)
        self.ppButton.show()
        self.buttonsLayout.addItem(self.stopButton)
        self.stopButton.show()
        self.buttonsLayout.addItem(self.nextButton)
        self.nextButton.show()
        self.applet.adjustSize()

    def removeButtons(self):
        self.buttonsLayout.removeItem(self.prevButton)
        self.prevButton.hide()
        self.buttonsLayout.removeItem(self.ppButton)
        self.ppButton.hide()
        self.buttonsLayout.removeItem(self.stopButton)
        self.stopButton.hide()
        self.buttonsLayout.removeItem(self.nextButton)
        self.nextButton.hide()
        self.buttonsLayout.addItem(self.connectButton)
        self.connectButton.show()
        self.applet.adjustSize()
    
    def removeIntarface(self):
        self.buttonsLayout.removeItem(self.prevButton)
        self.prevButton.hide()
        self.buttonsLayout.removeItem(self.ppButton)
        self.ppButton.hide()
        self.buttonsLayout.removeItem(self.stopButton)
        self.stopButton.hide()
        self.buttonsLayout.removeItem(self.nextButton)
        self.nextButton.hide()
        try:
            self.buttonsLayout.removeItem(self.connectButton)
            self.connectButton.hide()
        except:
            pass
        del self.prevButton
        del self.ppButton
        del self.stopButton
        del self.nextButton
        del self.connectButton
    
    def checkRunning(self):
        if mymoc.isRunning() == 0 and self.ppButton.isVisible():
            try:
                self.removeButtons();
            except:
                pass
        
# Button events    
    def startMocp(self):
        mymoc.StartMoc()
        
    def togglePlayback(self):
        if moc.get_state() == 0:
            moc.play()
        elif moc.get_state() == 1 or 2:
            moc.toggle_playback()

    def previous(self):
        if moc.get_state() == 1 or 2:
            moc.previous()

    def stop(self):
        if moc.get_state() == 1 or 2:
            moc.stop()

    def next(self):
        if moc.get_state() == 1 or 2:
            moc.next()

    def changeIcon(self):
        if moc.is_paused():
            self.ppButton.setIcon(self.pauseIcon)
        else:
            self.ppButton.setIcon(self.playIcon)

# Tooltip pupop window
    def hoverEnterEvent(self, event):
        if self.showpopup == 'true':
            self.updateToolTipInfo()
        else:
            Plasma.ToolTipManager.self().hide(self.applet)
    
    def updateToolTipInfo(self):
        self.data = Plasma.ToolTipContent()
        if mymoc.isRunning() == 1:
            if moc.get_state() == 0:
                maintext = "Moc player is running.\n Click play button for songs!"
                self.data.setMainText(maintext)
            elif moc.get_state() == 1 or 2:
                songinfo = moc.get_info_dict()
                maintext = "<font size = +2>" + songinfo['songtitle'] + "</font>" + " " + songinfo['timeleft']
                subtext = 'by <b>' + songinfo['artist'] + '</b> from <b>' +  songinfo['album'] + '<b>'
                unimaintext = maintext.decode('utf-8')
                unisubtext = subtext.decode('utf-8')
                self.data.setMainText(unimaintext)
                self.data.setSubText(unisubtext)
        else:
            maintext = "Moc player is not running.\n Click me to start it!"
            self.data.setMainText(maintext)

        Plasma.ToolTipManager.self().setContent(self.applet, self.data)
        
def CreateApplet(parent):
    return MocPlasmaClient(parent)
